/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/08/22.
//

#include "grid.hpp"
#include "fggl/entity/gridworld/zone.hpp"

namespace demo {

	GridScene::GridScene(fggl::App &app) : Game(app), m_grid(nullptr) {
	}

	void GridScene::activate() {
		Game::activate();
		fggl::debug::log(fggl::debug::Level::info, "GridScene::activate()");

		// create the grid world
		fggl::entity::grid::FloorTile empty{fggl::entity::grid::FloorTile::IMPOSSIBLE, fggl::gfx::colours::BLACK};
		fggl::entity::grid::FloorTile ground{1, fggl::gfx::colours::GREEN};
		m_tiles.m_floors.push_back(empty);
		m_tiles.m_floors.push_back(ground);

		fggl::entity::grid::WallTile noWall{};
		m_tiles.m_walls.push_back(noWall);

		m_grid = std::make_unique<fggl::entity::grid::Area2D<255,255>>(m_tiles);

		// create a small area to use
		for (int i=3; i<20; ++i) {
			auto& walls = m_grid->wallAt(i, 3, true);

			for (int j=3; j<20; ++j) {
				m_grid->setFloorAt(i, j, 1);
			}
		}

	}

	void GridScene::deactivate() {
		m_grid = nullptr;
	}


	constexpr float DRAW_SIZE = 64.0F;
	constexpr float DRAW_HALF = DRAW_SIZE / 2.0F;

	static void render_grid(fggl::gfx::Paint& paint, fggl::entity::grid::Area2D<255, 255>& grid) {
		for (int i=0; i <= 31; ++i) {
			for (int j=0; j <= 31; ++j) {
				auto& cell = grid.floorAt(i, j);
				fggl::math::vec2 drawPos{i * DRAW_SIZE, j*DRAW_SIZE};

				fggl::gfx::Path2D tileGfx = fggl::gfx::make_rect(drawPos, {DRAW_HALF, DRAW_HALF}, cell.colour);
				paint.fill(tileGfx);
			}
		}
	}

	float progress = 0.0f;
	void GridScene::render(fggl::gfx::Graphics &gfx) {
		Game::render(gfx);

		fggl::gfx::Paint paint;
		render_grid(paint, *m_grid);

		// draw test shapes to check grid alignment
		for (int sides = 3; sides <= 25; ++sides) {
			auto hexTest = fggl::gfx::make_shape(fggl::math::vec2{sides, 5} * DRAW_SIZE, DRAW_HALF, sides, {1.0F - (sides / 25.0F), 0.0f, sides / 25.0f});
			paint.fill(hexTest);
		}

		// draw with edges
		for (int sides = 3; sides <= 25; ++sides) {
			auto hexTest = fggl::gfx::make_shape(fggl::math::vec2{sides, 6} * DRAW_SIZE, DRAW_HALF, sides, {1.0F - (sides / 25.0F), 0.0f, sides / 25.0f});
			paint.stroke(hexTest);
		}

		// draw test arcs
		for (int sides = 0; sides < 12; ++sides) {
			float endAngle = progress * (M_PI * 2);

			float startAngle = sides/12.0F * (M_PI * 2);
			float endAngle2 = endAngle + startAngle;

			auto hexTest = fggl::gfx::make_arc(fggl::math::vec2{sides + 3, 7} * DRAW_SIZE, DRAW_HALF, startAngle, endAngle2, fggl::gfx::colours::CYAN);
			paint.fill(hexTest);
		}

		// draw sweep
		for (int sides = 2; sides <= 25; ++sides){
			float angle = progress * (M_PI * 2);
			float sliceSize = 1 / (float)sides * (M_PI * 2);

			float startAngle = angle;
			float endAngle = startAngle + sliceSize;

			auto hexTest = fggl::gfx::make_arc(fggl::math::vec2{sides + 3, 8} * DRAW_SIZE,
											   DRAW_HALF,
											   startAngle,
											   endAngle,
											   fggl::gfx::colours::CYAN);
			paint.fill(hexTest);
		}


		progress += 0.01F;
		if ( progress > 1.0F) {
			progress = 0.0F;
		}

		gfx.draw2D(paint);
	}

} // namespace demo
